@@ -51,7 +51,7 @@ class Session(object):
         self.client_id = client_id
         self.subscriptions = set()
         self.wildcard_subscriptions = set()
-        self.expiry_time = None
+        self.expiry_interval = 0
         self.client = None
         self.maximum_packet_size = MAXIMUM_PACKET_SIZE
         self.will_topic = None
@@ -60,7 +60,7 @@ class Session(object):
     def clean(self):
         self.subscriptions = set()
         self.wildcard_subscriptions = set()
-        self.expiry_time = None
+        self.expiry_interval = 0
         self.client = None
         self.maximum_packet_size = MAXIMUM_PACKET_SIZE
         self.will_topic = None
@@ -78,7 +78,7 @@ class Client(object):
         self._reader = reader
         self._writer = writer
         self._session = None
-        self._disconnect_reason = DisconnectReasonCode.NORMAL_DISCONNECTION
+        self._disconnect_reason = DisconnectReasonCode.UNSPECIFIED_ERROR
 
     async def serve_forever(self):
         addr = self._writer.get_extra_info('peername')
@@ -94,8 +94,10 @@ class Client(object):
                 raise ConnectError()
 
             await self.reader_loop()
-        except (ConnectError, DisconnectError):
+        except ConnectError:
             pass
+        except DisconnectError:
+            self._disconnect_reason = DisconnectReasonCode.NORMAL_DISCONNECTION
         except asyncio.IncompleteReadError:
             self.log_debug('Client connection lost.')
         except Exception as e:
@@ -112,10 +114,13 @@ class Client(object):
         if self._session is not None:
             self._session.client = None
 
-            # ToDo: Should not publish on normal disconnection.
             if self._session.will_topic is not None:
-                self._broker.publish(self._session.will_topic,
-                                     self._session.will_message)
+                if not self.is_normal_disconnection():
+                    self._broker.publish(self._session.will_topic,
+                                         self._session.will_message)
+
+            if self._session.expiry_interval == 0:
+                self._broker.remove_session(self._session.client_id)
 
         self.log_info('Closing client %r.', addr)
 
@@ -190,6 +195,10 @@ class Client(object):
         self._session.will_topic = will_topic
         self._session.will_message = will_message
 
+        if PropertyIds.SESSION_EXPIRY_INTERVAL in properties:
+            session_expiry_interval = properties[PropertyIds.SESSION_EXPIRY_INTERVAL]
+            self._session.expiry_interval = session_expiry_interval
+
         if (user_name is not None) or (password is not None):
             reason = ConnectReasonCode.BAD_USER_NAME_OR_PASSWORD
 
@@ -296,34 +305,66 @@ class Client(object):
             else:
                 LOGGER.info(f'{self._session.client_id} {fmt}', *args)
 
+    def is_normal_disconnection(self):
+        return self._disconnect_reason == DisconnectReasonCode.NORMAL_DISCONNECTION
+
 
 class Broker(object):
     """A limited MQTT version 5.0 broker.
 
-    `host` and `port` are the host and port to listen for clients on.
+    `host`, `port` and `secure_port` are the host and ports to listen
+    for clients on.
+
+    `secure_ssl` is an SSL context passed to `asyncio.start_server()`
+    as `ssl`.
 
     """
 
-    def __init__(self, host, port):
+    def __init__(self, host, port=1883, secure_port=8883, secure_ssl=None):
         self._host = host
         self._port = port
+        self._secure_port = secure_port
+        self._secure_ssl = secure_ssl
         self._sessions = {}
         self._subscribers = defaultdict(list)
         self._wildcard_subscribers = []
         self._listener = None
         self._listener_ready = asyncio.Event()
+        self._secure_listener = None
+        self._secure_listener_ready = asyncio.Event()
+        self._client_tasks = set()
 
     async def getsockname(self):
         await self._listener_ready.wait()
 
         return self._listener.sockets[0].getsockname()
 
+    async def secure_getsockname(self):
+        await self._secure_listener_ready.wait()
+
+        return self._secure_listener.sockets[0].getsockname()
+
     async def serve_forever(self):
         """Setup a listener socket and forever serve clients. This coroutine
         only ends if cancelled by the user.
 
         """
 
+        try:
+            await asyncio.gather(
+                self.insecure_serve_forever(),
+                self.secure_serve_forever()
+            )
+        except asyncio.CancelledError:
+            # Cancel all client tasks as the TCP server leaves them
+            # running.
+            for client_task in self._client_tasks:
+                client_task.cancel()
+
+            self._client_tasks = set()
+            raise
+
+    async def insecure_serve_forever(self):
         self._listener = await asyncio.start_server(self.serve_client,
                                                     self._host,
                                                     self._port)
@@ -335,9 +376,34 @@ class Broker(object):
         async with self._listener:
             await self._listener.serve_forever()
 
+    async def secure_serve_forever(self):
+        if self._secure_ssl is None:
+            return
+
+        self._secure_listener = await asyncio.start_server(self.serve_client,
+                                                           self._host,
+                                                           self._secure_port,
+                                                           ssl=self._secure_ssl)
+        self._secure_listener_ready.set()
+        listener_address = self._secure_listener.sockets[0].getsockname()
+
+        LOGGER.info('Listening for secure clients on %s.', listener_address)
+
+        async with self._secure_listener:
+            await self._secure_listener.serve_forever()
+
     async def serve_client(self, reader, writer):
+        current_task = asyncio.current_task()
+        self._client_tasks.add(current_task)
         client = Client(self, reader, writer)
-        await client.serve_forever()
+
+        try:
+            await client.serve_forever()
+        finally:
+            try:
+                self._client_tasks.remove(current_task)
+            except KeyError:
+                pass
 
     def add_subscriber(self, topic, session):
         topic_sessions = self._subscribers[topic]
@@ -398,6 +464,9 @@ class Broker(object):
 
         return session, session_present
 
+    def remove_session(self, client_id):
+        del self._sessions[client_id]
+
     def publish(self, topic, message):
         """Publish given topic and message to all subscribers.
 
