@@ -10,7 +10,6 @@ from unittest.mock import patch
 from io import StringIO
 
 import mqttools
-from mqttools import QoS
 
 
 HOST = 'localhost'
@@ -233,7 +232,7 @@ class MQTToolsTest(unittest.TestCase):
             '--port', str(self.broker.address[1]),
             '--client-id', 'mqttools_publish',
             '/a/b',
-            'apa'
+            '617061'
         ]
 
         stdout = StringIO()
@@ -403,6 +402,7 @@ class MQTToolsTest(unittest.TestCase):
         client = mqttools.Client(*self.broker.address,
                                  'bar',
                                  topic_alias_maximum=0,
+                                 connect_delays=[],
                                  keep_alive_s=0)
 
         with self.assertRaises(mqttools.ConnectError) as cm:
@@ -491,6 +491,7 @@ class MQTToolsTest(unittest.TestCase):
                                  'bar',
                                  session_expiry_interval=120,
                                  topic_alias_maximum=0,
+                                 connect_delays=[],
                                  keep_alive_s=0)
 
         with self.assertRaises(mqttools.SessionResumeError):
